# !/usr/bin/env python
# encoding:UTF-8

from django.conf import settings, global_settings as default_settings
from django.core.management import call_command
from django.test.utils import get_runner
from os.path import dirname, realpath
import django
import sys
import os
from dynamic_preferences.registries import autodiscover

# Give feedback on used versions
sys.stderr.write('Using Python version {0} from {1}\n'.format(sys.version[:5], sys.executable))
sys.stderr.write('Using Django version {0} from {1}\n'.format(
    django.get_version(),
    os.path.dirname(os.path.abspath(django.__file__)))
)

# Detect location and available modules
module_root = dirname(realpath(__file__))
sys.path.append(module_root)

# Inline settings file
settings.configure(
    DEBUG=False,  # will be False anyway by DjangoTestRunner.
    TEMPLATE_DEBUG=False,
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:'
        }
    },
    MIDDLEWARE_CLASSES=(
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    ),
    TEMPLATE_CONTEXT_PROCESSORS=default_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'django.core.context_processors.request',
        'dynamic_preferences.processors.global_preferences',
        'dynamic_preferences.processors.user_preferences',
    ),
    INSTALLED_APPS=(
        'django.contrib.auth',
        'django.contrib.sites',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
        'dynamic_preferences',
        'tests.test_app'
    ),
    ROOT_URLCONF='tests.urls',
    SITE_ID=1,
    STATIC_URL="/static/",
    TEMPLATE_LOADERS=(
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    ),

    DYNAMIC_PREFERENCES_USE_TEST_PREFERENCES=True,
    LOGGING={
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'simple': {
                'format': '%(levelname)s %(message)s'
            },
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'simple'
            },
        },
        'loggers': {
            'django.request': {
                'handlers': ['console'],
                'propagate': True,
                'level': 'DEBUG',
            },
        },
    },
    TESTING=True,
    TEST_RUNNER='django_nose.NoseTestSuiteRunner',
)

django.setup()
call_command('syncdb', verbosity=1, interactive=False)
autodiscover()

# ---- app start
verbosity = 2 if '-v' in sys.argv else 1

TestRunner = get_runner(settings)  # DjangoTestSuiteRunner
runner = TestRunner(verbosity=verbosity, interactive=True, failfast=False)
failures = runner.run_tests(['tests'])

if failures:
    sys.exit(bool(failures))



# !/usr/bin/env python
# encoding:UTF-8

import operator
from django.contrib import admin
from dynamic_preferences.models import GlobalPreferenceModel, UserPreferenceModel
from django import forms
from dynamic_preferences.types import FilePreference
from django.db import models
from django.contrib.admin.util import lookup_needs_distinct
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _


class PreferenceChangeListForm(forms.ModelForm):
    """
    A form that integrate dynamic-preferences into django.contrib.admin
    """
    # Me must use an acutal model field, so we use raw_value. However, 
    # instance.value will be displayed in form.
    raw_value = forms.CharField()

    def is_multipart(self):
        """
        Returns True if the form needs to be multipart-encoded, i.e. it has
        FileInput. Otherwise, False.
        """
        for section in self.instance.registry:
            for pref in self.instance.registry[section]:
                if isinstance(self.instance.registry[section][pref], FilePreference):
                    return True
        return False

    def __init__(self, *args, **kwargs):
        self.instance = kwargs.get('instance')
        super(PreferenceChangeListForm, self).__init__(*args, **kwargs)
        self.fields['raw_value'] = self.instance.preference.setup_field()

    def save(self, *args, **kwargs):
        ikargs = {}
        if isinstance(self.instance.preference, FilePreference):
            ikargs['delete_filename'] = self.initial['raw_value']
        self.cleaned_data['raw_value'] = self.instance.preference.serializer.serialize(self.cleaned_data['raw_value'],
                                                                                       **ikargs)
        return super(PreferenceChangeListForm, self).save(*args, **kwargs)


class GlobalPreferenceChangeListForm(PreferenceChangeListForm):
    class Meta:
        model = GlobalPreferenceModel


class UserPreferenceChangeListForm(PreferenceChangeListForm):
    class Meta:
        model = UserPreferenceModel


class DynamicPreferenceAdmin(admin.ModelAdmin):
    changelist_form = PreferenceChangeListForm
    readonly_fields = ('section', 'name', 'value', 'help')
    fields = ("raw_value",)
    list_display = ['section', 'name', 'raw_value', 'help']
    list_display_links = ['name', ]
    list_editable = ('raw_value',)
    search_fields = ['section', 'name', 'help']
    list_filter = ('section',)
    ordering = ('section', 'name')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_actions(self, request):
        return []

    def get_changelist_form(self, request, **kwargs):
        return self.changelist_form

    def get_search_results(self, request, queryset, search_term):

        # Apply keyword searches.
        def construct_search(field_name):
            if field_name.startswith('^'):
                return "%s__istartswith" % field_name[1:]
            elif field_name.startswith('='):
                return "%s__iexact" % field_name[1:]
            elif field_name.startswith('@'):
                return "%s__search" % field_name[1:]
            else:
                return "%s__icontains" % field_name

        initial_queryset = queryset
        use_distinct = False
        if self.search_fields and search_term:
            orm_lookups = [construct_search(str(search_field)) for search_field in self.search_fields]
            for bit in search_term.split():
                or_queries = [models.Q(**{orm_lookup: bit}) for orm_lookup in orm_lookups]
                queryset = queryset.filter(reduce(operator.or_, or_queries))
            if not use_distinct:
                for search_spec in orm_lookups:
                    if lookup_needs_distinct(self.opts, search_spec):
                        use_distinct = True
                        break
        if not queryset.count():
            queryset = initial_queryset
            if search_term != '':
                self.message_user(request,
                                  _('No results found for \'%(search_term)s\'') % {'search_term': search_term},
                                  messages.INFO)
        return queryset, use_distinct


class GlobalPreferenceAdmin(DynamicPreferenceAdmin):
    form = GlobalPreferenceChangeListForm
    changelist_form = GlobalPreferenceChangeListForm


admin.site.register(GlobalPreferenceModel, GlobalPreferenceAdmin)


class UserPreferenceAdmin(DynamicPreferenceAdmin):
    form = UserPreferenceChangeListForm
    changelist_form = UserPreferenceChangeListForm
    list_display = ['user'] + DynamicPreferenceAdmin.list_display
    search_fields = ['user__username'] + DynamicPreferenceAdmin.search_fields


admin.site.register(UserPreferenceModel, UserPreferenceAdmin)



# !/usr/bin/env python
# encoding:UTF-8

import os
from six import string_types
from django.conf import settings
from django.core.files.storage import default_storage
from fields import FieldUpfile
from django.template import defaultfilters
from dynamic_preferences.settings import preferences_settings
from django.utils import timezone


class SerializationError(Exception):
    pass


class BaseSerializer(object):
    """
    A serializer take a Python variable and returns a string that can be stored safely in database
    """
    exception = SerializationError

    @classmethod
    def serialize(cls, value, **kwargs):
        """
        Return a string from a Python var
        """
        raise NotImplementedError

    @classmethod
    def deserialize(cls, value, **kwargs):
        """
            Convert a python string to a var
        """
        raise NotImplementedError


class BooleanSerializer(BaseSerializer):
    true = (
        "True",
        "true",
        "TRUE",
        "1",
        "YES",
        "Yes",
        "yes",
    )

    false = (
        "False",
        "false",
        "FALSE",
        "0",
        "No",
        "no",
        "NO",
    )

    @classmethod
    def serialize(cls, value, **kwargs):
        if value:
            return True
        else:
            return False

    @classmethod
    def deserialize(cls, value, **kwargs):
        if value in cls.true:
            return True
        elif value in cls.false:
            return False
        else:
            raise cls.exception("Value {0} can't be deserialized to a Boolean".format(value))


class IntSerializer(BaseSerializer):
    @classmethod
    def serialize(cls, value, **kwargs):
        if not isinstance(value, int):
            raise cls.exception('IntSerializer can only serialize int values')

        return value.__str__()

    @classmethod
    def deserialize(cls, value, **kwargs):
        try:
            return int(value)
        except:
            raise cls.exception("Value {0} cannot be converted to int")


class StringSerializer(BaseSerializer):
    @classmethod
    def serialize(cls, value, **kwargs):
        if not isinstance(value, string_types):
            raise cls.exception("Cannot serialize, value {0} is not a string".format(value))
        if kwargs.get("escape_html", False):
            return defaultfilters.force_escape(value.encode('utf-8'))
        else:
            return value.encode('utf-8')

    @classmethod
    def deserialize(cls, value, **kwargs):
        """String deserialisation just return the value as a string"""
        try:
            return str(value.encode('utf-8'))
        except:
            raise cls.exception("Cannot deserialize value {0} tostring".format(value))


class UnsetValue(object):
    pass


UNSET = UnsetValue()


class FileSerializer(BaseSerializer):
    @classmethod
    def serialize(cls, value, **kwargs):
        """
        Return a string from a Python var
        """
        return cls.to_db(value, **kwargs)

    @classmethod
    def deserialize(cls, value, **kwargs):
        """
            Convert a python string to a var
        """
        return cls.to_python(value, **kwargs)

    @classmethod
    def clean_to_db_value(cls, value):
        return value

    @staticmethod
    def handle_uploaded_file(f, path):
        # create folders for upload_to or app dir if necessary
        try:
            os.makedirs(os.path.dirname(path))
        except OSError:
            if not os.path.isdir(os.path.dirname(path)):
                raise

        with open(path, 'wb+') as destination:
            for chunk in f.chunks():
                destination.write(chunk)

    @staticmethod
    def append_suffix(filename):
        # add random suffix to avoid browser cache
        name, ext = os.path.splitext(filename)
        return "{name}_{suffix}{ext}".format(name=name, suffix=timezone.now().strftime('%Y%m%d%H%M%S%f')[-4:], ext=ext)

    @classmethod
    def to_db(cls, dfile, **kwargs):
        # to_db is passed a file object from forms.FileField
        if not settings.MEDIA_ROOT:
            raise cls.exception("You need to set MEDIA_ROOT in your settings.py")
        try:
            dfile.name = cls.append_suffix(dfile.name)
            path = os.path.join(settings.MEDIA_ROOT, preferences_settings.FILE_PREFERENCE_REL_UPLOAD_DIR,
                                dfile.name)
            cls.handle_uploaded_file(dfile, path)
        except AttributeError:
            return ''
        if kwargs.get('delete_filename', None):
            # delete previous file
            try:
                os.remove(os.path.join(settings.MEDIA_ROOT, preferences_settings.FILE_PREFERENCE_REL_UPLOAD_DIR,
                                       kwargs.get('delete_filename')))
            except OSError:
                pass
        return dfile.name

    @classmethod
    def to_python(cls, value, **kwargs):
        filename = value
        if not settings.MEDIA_ROOT:
            raise cls.exception("You need to set MEDIA_ROOT in your settings.py")

        path = os.path.join(settings.MEDIA_ROOT, preferences_settings.FILE_PREFERENCE_REL_UPLOAD_DIR, filename)

        if os.path.isfile(path):
            # https://yuji.wordpress.com/2013/01/30/django-form-field-in-initial-data-requires-a-fieldfile-instance/
            # TODO: Understand this FieldFile better and maybe remove the FakeField workaround
            class FakeField(object):
                storage = default_storage

            fieldfile = FieldUpfile(None, FakeField, filename)
            return fieldfile
        else:
            return None



# !/usr/bin/env python
# encoding:UTF-8

from django.core.management.base import BaseCommand
from dynamic_preferences.models import GlobalPreferenceModel, UserPreferenceModel, SitePreferenceModel
try:
    from django.contrib.auth import get_user_model
except ImportError:  # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()
from django.contrib.sites.models import Site
from dynamic_preferences.registries import autodiscover, global_preferences_registry, user_preferences_registry, \
    site_preferences_registry


def delete_preferences(queryset):
    """
    Delete preferences objects if they are not present in registry. Return a list of deleted objects
    """
    deleted = []
    # Iterate through preferences. If an error is raised when accessing preference object, just delete it
    for p in queryset:
        try:
            p.preference
        except KeyError:
            p.delete()
            deleted.append(p)
    return deleted


class Command(BaseCommand):
    help = "Find and delete preferences from database if they don't exist in registries. Create preferences that are " \
           "not present in database"

    def handle(self, *args, **options):
        autodiscover()
        deleted = delete_preferences(GlobalPreferenceModel.objects.all())
        print "Deleted %s global preference models : %s" % \
            (len(deleted), ", ".join(['Section: %s - Name: %s' % (p.section, p.name) for p in deleted]))

        deleted = delete_preferences(UserPreferenceModel.objects.all())
        print "Deleted %s user preference models : %s" % \
            (len(deleted), ", ".join(['Section: %s - Name: %s' % (p.section, p.name) for p in deleted]))

        deleted = delete_preferences(SitePreferenceModel.objects.all())
        print "Deleted %s site preference models : %s" % \
            (len(deleted), ", ".join(['Section: %s - Name: %s' % (p.section, p.name) for p in deleted]))

        # Create needed preferences
        # Global
        preferences = global_preferences_registry.preferences()
        for p in preferences:
            p.to_model(**{"help": p.help})
        print 'Created/updated %s default global preferences' % len(preferences)

        # User
        preferences = user_preferences_registry.preferences()
        users = list(User.objects.all())
        for user in users:
            for p in preferences:
                p.to_model(user=user, **{"help": p.help})
        print 'Created/updated default preferences for %s users' % len(users)

        # Site
        preferences = site_preferences_registry.preferences()
        try:
            site = Site.objects.get(pk=1)
        except Site.DoesNotExist:
            print 'Cannot create default preference for first site. Please create add at least one site in your ' \
                  'database.'
            site = None
        if site is not None:
            for p in preferences:
                p.to_model(site=site, **{"help": p.help})
            print 'Created/updated %s default preferences for first site' % len(preferences)



# !/usr/bin/env python
# encoding:UTF-8

import collections
import os
from django.conf import settings
from dynamic_preferences.settings import preferences_settings


def update(d, u):
    """
        Custom recursive update of dictionary
        from http://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth
    """
    for k, v in u.iteritems():
        if isinstance(v, collections.Mapping):
            r = update(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d


def get_default_file_path(filename):
    return os.path.join(settings.PROJECT_DIR, preferences_settings.FILE_PREFERENCE_REL_DEFAULT_DIR, filename)


def get_upload_file_path(filename):
    return os.path.join(settings.MEDIA_ROOT, preferences_settings.FILE_PREFERENCE_REL_UPLOAD_DIR, filename)

# def get_static_dynamic_prefereces_dir():
#     from distutils.sysconfig import get_python_lib
#     return get_python_lib() + 'django-dynamic-preferences/django_preferences/media',



# !/usr/bin/env python
# encoding:UTF-8

from django.conf.urls import patterns, url
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from . import views
from .registries import global_preferences_registry
from .forms import GlobalPreferenceForm


urlpatterns = patterns(
    '',
    url(r'^global/$',
        staff_member_required(views.PreferenceFormView.as_view(registry=global_preferences_registry,
                                                               form_class=GlobalPreferenceForm)),
        name="dynamic_preferences.global"),

    url(r'^global/(?P<section>\w+)$', 
        staff_member_required(views.PreferenceFormView.as_view(registry=global_preferences_registry,
                                                               form_class=GlobalPreferenceForm)),
        name="dynamic_preferences.global.section"),

    url(r'^user/$', login_required(views.UserPreferenceFormView.as_view()), name="dynamic_preferences.user"),

    url(r'^user/(?P<section>\w+)$', login_required(views.UserPreferenceFormView.as_view()),
        name="dynamic_preferences.user.section"),
)



# !/usr/bin/env python
# encoding:UTF-8

import os
import urlparse
from django.conf import settings
from dynamic_preferences.settings import preferences_settings
from django.db.models.fields.files import FieldFile


class FieldUpfile(FieldFile):

    @staticmethod
    def get_file_url(name):
        return urlparse.urljoin(settings.MEDIA_URL,
                                os.path.join(preferences_settings.FILE_PREFERENCE_REL_UPLOAD_DIR, name))

    @property
    def url(self):
        return self.get_file_url(self.name)



from django.conf.urls import patterns, include, url
from dynamic_preferences import views
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^', include("dynamic_preferences.urls")), (r'^admin/', include(admin.site.urls)),
    url(r'^test/template$', views.RegularTemplateView.as_view(), name="dynamic_preferences.test.templateview"),
)



# !/usr/bin/env python
# encoding:UTF-8

import os
from setuptools import setup, find_packages
import dynamic_preferences

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()
VERSION = dynamic_preferences.__version__

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-dynamic-preferences-plus',
    packages=find_packages(),
    version=VERSION,
    description='A django app for registering dynamic global, site and user preferences',
    long_description=README,
    author='Team QWL',
    author_email='padova@quag.com',
    url='https://github.com/mirk8xr/django-dynamic-preferences',
    download_url='https://github.com/mirk8xr/django-dynamic-preferences/archive/%s.tar.gz' % VERSION,
    include_package_data=True,
    license='BSD',
    zip_safe=False,
    keywords=['django', 'preferences', 'admin'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        "django<1.7",
        "six",
    ],
)



# !/usr/bin/env python
# encoding:UTF-8

from django import forms
from dynamic_preferences.fields import FieldUpfile
from .registries import global_preferences_registry, user_preferences_registry, site_preferences_registry
from six import string_types
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe, SafeText
from django.utils.translation import ugettext_lazy
from django.utils.html import format_html, force_text
from django.forms.util import flatatt


def preference_form_builder(form_base_class, preferences=(), **kwargs):
    """
    Return a form class for updating preferences
    :param form_base_class: a Form class used as the base. Must have a ``registry` attribute
    :param preferences: a list of :py:class:
    :var section: a section where the form builder will load preferences
    """
    registry = form_base_class.registry
    preferences_obj = []
    if len(preferences) > 0:
        # Preferences have been selected explicitly 
        for pref in preferences:
            if isinstance(pref, string_types):
                preferences_obj.append(registry.get(name=pref))
            elif type(pref) == tuple:
                preferences_obj.append(registry.get(name=pref[0], section=pref[1]))
            else:
                raise NotImplementedError("The data you provide can't be converted to a Preference object")
    else:
        # Try to use section param
        preferences_obj = registry.preferences(section=kwargs.get('section', None))

    fields = {}
    instances = []
    for preference in preferences_obj:
        f = preference.field
        model_kwargs = kwargs.get('model', {})
        instance = preference.to_model(**model_kwargs)
        f.initial = instance.value
        fields[preference.identifier()] = f
        instances.append(instance)

    form_class = type('Custom' + form_base_class.get_name(), (form_base_class,), {})
    form_class.base_fields = fields
    form_class.preferences = preferences_obj
    form_class.instances = instances
    return form_class


def global_preference_form_builder(preferences=(), **kwargs):
    """
    A shortcut :py:func:`preference_form_builder(GlobalPreferenceForm, preferences, **kwargs)`
    """
    return preference_form_builder(GlobalPreferenceForm, preferences, **kwargs)


def user_preference_form_builder(user, preferences=(), **kwargs):
    """
    A shortcut :py:func:`preference_form_builder(UserPreferenceForm, preferences, **kwargs)`
    :param user: a :py:class:`django.contrib.auth.models.User` instance
    :param preferences: preferences
    """
    return preference_form_builder(UserPreferenceForm, preferences, model={'user': user}, **kwargs)


def site_preference_form_builder(preferences=(), **kwargs):
    """
    A shortcut :py:func:`preference_form_builder(SitePreferenceForm, preferences, **kwargs)`
    """
    return preference_form_builder(SitePreferenceForm, preferences, **kwargs)


class PreferenceForm(forms.Form):
    registry = None

    @classmethod
    def get_name(cls):
        return cls.__name__

    def update_preferences(self):
        for instance in self.instances:
            instance.value = self.cleaned_data[instance.preference.identifier()]
            instance.save()


class GlobalPreferenceForm(PreferenceForm):
    registry = global_preferences_registry


class UserPreferenceForm(PreferenceForm):
    registry = user_preferences_registry


class SitePreferenceForm(PreferenceForm):
    registry = site_preferences_registry


class OptimisedClearableFileInput(forms.ClearableFileInput):
    template_with_initial = (
        '<div><div style="width: 50px;max-height: 50px;float: left;margin: 0px 10px 0px 0px;">'
        '<img src="%(initial_url)s" style=" max-width:50px; max-height:50px;" /></div> '
        '<div style="">%(initial_text)s: <a href="%(initial_url)s" target="_blank">%(initial)s</a></div> </div>'
        # '<span class="clear-file"> %(clear_template)s</span> <span>%(input_text)s: %(input)s </span>'
        '<span>%(input)s</span>'
    )
    clear_checkbox_label = ugettext_lazy('Remove this file')
    template_with_clear = '%(clear)s %(clear_checkbox_label)s -'

    @staticmethod
    def is_initial(value):
        """
        Return whether value is considered to be initial value.
        """
        return bool(value)

    @staticmethod
    def get_template_substitution_values(value):
        """
        Return value-related substitutions.
        """
        return {
            'initial': conditional_escape(value),
            'initial_url': FieldUpfile.get_file_url(conditional_escape(value)),
        }

    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': 'File name',
            'input_text': 'File',
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = '%(input)s'
        substitutions['input'] = super(OptimisedClearableFileInput, self).render(name, value, attrs)

        if self.is_initial(value):
            template = self.template_with_initial
            substitutions.update(self.get_template_substitution_values(value))
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions['clear_checkbox_name'] = conditional_escape(checkbox_name)
                substitutions['clear_checkbox_id'] = conditional_escape(checkbox_id)
                substitutions['clear'] = forms.CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
                substitutions['clear_template'] = self.template_with_clear % substitutions
        return mark_safe(template % substitutions)


class ColorInput(forms.TextInput):

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_text(self._format_value(value))

        t = "<input id='" + name + "-sp' style='position:relative;left: 5px;width: 80px;' maxlength='7' value='" + \
            value + "' />"
        js_script = "<script type='text/javascript'>"
        js_script = js_script + "document.getElementsByName('" + name + "')[0]" \
                                ".addEventListener('input', function(e){ " \
                                "document.getElementById('" + name + "-sp').value = this.value; });"
        js_script = js_script + "document.getElementById('" + name + "-sp')" \
                                ".addEventListener('input', function(e){ console.log(this.value); " \
                                "document.getElementsByName('" + name + "')[0].value = this.value; });</script>"
        html = format_html('<input{0} style="width:65px;"/>' + t, flatatt(final_attrs)) + SafeText(js_script)
        return html



# !/usr/bin/env python
# encoding:UTF-8

"""
Preference models, queryset and managers that handle the logic for persisting preferences.
"""
from django.db import models
try:
    from django.contrib.auth import get_user_model
except ImportError:  # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()

from django.contrib.sites.models import Site
from django.db.models.signals import post_save
from django.conf import settings
from django.utils.functional import cached_property
from dynamic_preferences.registries import user_preferences_registry, site_preferences_registry, \
    global_preferences_registry
from django.core.files import File
from django.utils.translation import ugettext as _
from dynamic_preferences.types import FilePreference
from dynamic_preferences.utils import get_default_file_path


class PreferenceModelManager(models.Manager):
    def to_dict(self, **kwargs):
        """ 
            Return a dict of preference models values with the same structure as registries
            Used to access preferences value within templates
        """

        preferences = self.get_queryset().all()
        if kwargs:
            preferences = preferences.filter(**kwargs)

        d = {}
        for p in preferences:
            try:
                d[p.section][p.name] = p.value
            except KeyError:
                d[p.section] = {}
                d[p.section][p.name] = p.value

        return d


class BasePreferenceModel(models.Model):
    """
    A base model with common logic for all preferences models.
    """

    #: The section under which the preference is declared
    section = models.CharField(max_length=255, db_index=True, blank=True, null=True, default=None)

    #: a name for the preference
    name = models.CharField(max_length=255, db_index=True)

    #: a value, serialized to a string. This field should not be accessed directly,
    # use :py:attr:`BasePreferenceModel.value` instead
    raw_value = models.TextField(null=True, blank=True)

    #: Keep a reference to the whole preference registry.
    #: In order to map the Preference Model Instance to the Preference object.
    registry = None

    #: help text
    help = models.TextField(null=True, blank=True, default="")

    objects = PreferenceModelManager()

    class Meta:
        abstract = True
        app_label = 'dynamic_preferences'

    def __init__(self, *args, **kwargs):
        # Check if the model is already saved in DB

        v = kwargs.pop("value", None)
        super(BasePreferenceModel, self).__init__(*args, **kwargs)

        new = self.pk is None

        if new:
            if v is not None:
                if isinstance(self.registry[kwargs['section']][kwargs['name']], FilePreference):
                    # path = get_upload_file_path(v)
                    # if not os.path.exists(os.path.dirname(path)):
                    #     os.makedirs(os.path.dirname(path))
                    # shutil.copy2(get_default_file_path(v), path)
                    # self.value = open(path, 'wb+')
                    self.value = File(open(get_default_file_path(v), 'rb+'), name=v)
                else:
                    self.value = v
            else:
                self.value = self.preference.default

    @cached_property
    def preference(self):
        # return self.registry[self.section][self.name].initial
        # r = None
        # try:
        #     r = self.registry.get(section=self.section, name=self.name)
        # except:
        #     r = {}
        # return r
        return self.registry.get(section=self.section, name=self.name)

    def set_value(self, value):
        """
            Save serialized self.value to self.raw_value
        """
        self.raw_value = self.preference.serializer.serialize(value)

    def get_value(self):
        """
            Return deserialized self.raw_value
        """
        return self.preference.serializer.deserialize(self.raw_value)

    value = property(get_value, set_value)

    def __unicode__(self):
        return self.preference.identifier()


class GlobalPreferenceModel(BasePreferenceModel):
    registry = global_preferences_registry

    class Meta:
        unique_together = ('section', 'name')
        app_label = 'dynamic_preferences'
        verbose_name = _('global preference')
        verbose_name_plural = _('global preferences')


class UserPreferenceModel(BasePreferenceModel):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="preferences")
    registry = user_preferences_registry

    class Meta:
        unique_together = ('user', 'section', 'name')
        app_label = 'dynamic_preferences'
        verbose_name = _('user preference')
        verbose_name_plural = _('user preferences')


class SitePreferenceModel(BasePreferenceModel):
    site = models.ForeignKey(Site, related_name="preferences")
    registry = site_preferences_registry

    class Meta:
        unique_together = ('site', 'section', 'name')
        app_label = 'dynamic_preferences'
        verbose_name = _('site preference')
        verbose_name_plural = _('site preferences')


global_preferences = GlobalPreferenceModel.objects
site_preferences = SitePreferenceModel.objects
user_preferences = UserPreferenceModel.objects


# Create default preferences for new users
# Right now, only works if the model is django.contrib.auth.models.User
# And if settings.CREATE_DEFAULT_PREFERENCES_FOR_NEW_USERS is set to True in settings
def create_default_preferences(sender, **kwargs):
    create_default_preferencesfor_new_users = getattr(settings, 'CREATE_DEFAULT_PREFERENCES_FOR_NEW_USERS', True)
    if create_default_preferencesfor_new_users and settings.AUTH_USER_MODEL == "auth.User":
        # the object which is saved can be accessed via kwargs 'instance' key.
        obj = kwargs['instance']
        # created = kwargs.get("created")
        user_preferences_registry.create_default_preferences(obj)


post_save.connect(create_default_preferences, sender=User)
